﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Xml;

namespace TransformToDotnetcore
{
    class Program
    {
        static void Main(string[] args)
        {
            while (true)
            {
                //accept input

                Console.Write("请输入项目的完整路径（按‘q’退出）：");
                string inputStr = Console.ReadLine().Trim();
                if (inputStr == "q")
                {
                    break;
                }

                //get path 

                string projectPath = inputStr.TrimEnd('\\');
                if (!Directory.Exists(projectPath))
                {
                    Console.WriteLine("未查找到匹配路径");
                    continue;
                }

                //get .sln files

                string[] slnFiles = Directory.GetFiles(projectPath, "*.sln", System.IO.SearchOption.AllDirectories);
                if (slnFiles.Length == 0)
                {
                    projectPath = projectPath.Substring(0, projectPath.LastIndexOf('\\'));
                    slnFiles = Directory.GetFiles(projectPath, "*.sln");
                    if (slnFiles.Length == 0)
                    {
                        Console.WriteLine("未查找到匹配的解决方案路径");
                        continue;
                    }
                }

                //Transform

                if (slnFiles.Length > 1)
                {
                    Console.WriteLine("查询到您的目录中包含多个项目，您确定全部迁移吗？（y/n）");
                    string key = Console.ReadLine().Trim().ToLower();
                    if (key == "n" || key == "no")
                    {
                        continue;
                    }
                }
                Transform(slnFiles);
            }
        }

        private static void Transform(string[] slnFiles)
        {
            for (int i = 0; i < slnFiles.Length; i++)
            {
                //get projectguid

                string[] slnLine = File.ReadAllLines(slnFiles[i]);
                List<string> projectGuid = new List<string>();
                for (int k = 0; k < slnLine.Length; k++)
                {
                    if (slnLine[k].Contains(".csproj") && slnLine[k].Contains("Project(\"{"))
                    {
                        projectGuid.Add(Regex.Match(slnLine[k], "Project\\(\"{([\\w\\-]+)}").Groups[1].Value);
                    }
                }
                projectGuid = projectGuid.Distinct().ToList();

                //edited .sln

                StreamReader slnSr = new StreamReader(slnFiles[i], System.Text.Encoding.UTF8);
                string slnContent = slnSr.ReadToEnd();
                slnSr.Close();

                //replace csproj
                slnContent = slnContent.Replace(".csproj", ".xproj");

                //replace project
                for (int l = 0; l < projectGuid.Count; l++)
                {
                    slnContent = slnContent.Replace(projectGuid[l], Guid.NewGuid().ToString());
                }

                File.WriteAllText(slnFiles[i], slnContent, System.Text.Encoding.UTF8);

                //get all csprojPath

                string projectPath = slnFiles[i].Substring(0, slnFiles[i].LastIndexOf('\\'));
                string[] csprojFiles = System.IO.Directory.GetFiles(projectPath, "*.csproj", System.IO.SearchOption.AllDirectories);
                for (int j = 0; j < csprojFiles.Length; j++)
                {
                    int csprojFileIndex = csprojFiles[j].LastIndexOf('\\');
                    string csprojPath = csprojFiles[j].Substring(0, csprojFileIndex + 1);
                    string nameSpace = csprojFiles[j].Substring(csprojFileIndex + 1, csprojFiles[j].Length - csprojFileIndex - 8);

                    //add files
                    AddFiles(nameSpace, csprojPath);

                    Console.WriteLine(csprojPath);
                    File.Delete(csprojFiles[j]);
                }
            }
        }


        private static void AddFiles(string nameSpace, string csprojPath)
        {
            //get importent infomation

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(csprojPath + $"{nameSpace}.csproj"); 
            XmlNamespaceManager nsMgr = new XmlNamespaceManager(xmlDoc.NameTable); nsMgr.AddNamespace("ns", "http://schemas.microsoft.com/developer/msbuild/2003");

            //projectGuid

            string projectGuid = xmlDoc.SelectSingleNode("//ns:ProjectGuid", nsMgr).InnerText;
            string xprojContent = System.IO.File.ReadAllText(Directory.GetCurrentDirectory() + "\\xproj.xml")
                  .Replace("{RootNamespace}", nameSpace)
                  .Replace("{ProjectGuid}", projectGuid);
            System.IO.File.WriteAllText(csprojPath + $"{nameSpace}.xproj", xprojContent, System.Text.Encoding.UTF8);

            //using project

            XmlNodeList xnl = xmlDoc.SelectNodes("//ns:ProjectReference",nsMgr);
            List<string> projetList = new List<string>();
            foreach (XmlNode item in xnl)
            {
                projetList.Add(item.SelectSingleNode("ns:Name", nsMgr).InnerText);
            }

            //choose project
            
            string csprojContent = xmlDoc.InnerXml;
            string projectFile = "project.json";
            if (csprojContent.Contains("xunit."))
            {
                projectFile = "testProject.json";
            }
            else if (csprojContent.Contains("Microsoft.AspNet.WebApi"))
            {
                projectFile = "webapiProject.json";
            }
            else if (File.Exists(csprojPath + "web.config"))
            {
                projectFile = "mvcProject.json";

                //copy bower.json
                File.Delete(csprojPath + "bower.json");
                System.IO.File.Copy(Directory.GetCurrentDirectory() + "\\bower.json", csprojPath + "bower.json");
            }

            //edited project.json

            string projectContent = System.IO.File.ReadAllText(Directory.GetCurrentDirectory() + $"\\{projectFile}");
            if (projetList.Count > 0)
            {
                StringBuilder projectSB = new StringBuilder();
                foreach (string item in projetList)
                {
                    projectSB.Append($"\"{item}\":\"1.0.0-*\",\n");
                }
                projectContent = projectContent.Replace("$ProjectReference$", projectSB.ToString());
            }
            else
            {
                projectContent = projectContent.Replace("$ProjectReference$", "");
            }
            File.Delete(csprojPath + "project.json");
            System.IO.File.WriteAllText($"{csprojPath}\\project.json", projectContent, System.Text.Encoding.UTF8);

            if (File.Exists(csprojPath + "web.config"))
            {
                File.Delete(csprojPath + "hosting.json");
                File.Delete(csprojPath + "appsettings.json");
                File.Delete(csprojPath + "appsettings.Development.json");

                System.IO.File.Copy(Directory.GetCurrentDirectory() + "\\hosting.json", csprojPath + "hosting.json");
                System.IO.File.Copy(Directory.GetCurrentDirectory() + "\\appsettings.json", csprojPath + "appsettings.json");
                System.IO.File.Copy(Directory.GetCurrentDirectory() + "\\appsettings.Development.json", csprojPath + "appsettings.Development.json");

                string programContent = System.IO.File.ReadAllText(Directory.GetCurrentDirectory() + "\\Program.txt").Replace("{RootNamespace}", nameSpace);
                System.IO.File.WriteAllText(csprojPath + "Program.cs", programContent, System.Text.Encoding.UTF8);

                string startupContent = System.IO.File.ReadAllText(Directory.GetCurrentDirectory() + "\\Startup.txt").Replace("{RootNamespace}", nameSpace);
                System.IO.File.WriteAllText(csprojPath + "Startup.cs", programContent, System.Text.Encoding.UTF8);
            }
        }
    }
}